<?php
session_start();
include 'install.php';
require_once 'db.php';

// Assign an admin ID if not set.
if (!isset($_SESSION['admin_id'])) {
    try {
        $_SESSION['admin_id'] = uniqid('admin_', true);
    } catch (PDOException $e) {
        die("Failed to assign admin ID: " . $e->getMessage());
    }
}
$adminId = $_SESSION['admin_id'];
?>
<!DOCTYPE html>
<html>
<head>
    <title>Admin Call Center</title>
    <style>
        table { border-collapse: collapse; width: 100%; }
        th, td { border: 1px solid #ccc; padding: 8px; text-align: left; }
        th { background-color: #f2f2f2; }
    </style>
</head>
<body>
<h1>Admin Call Center</h1>
<p>Your Admin ID: <span id="adminIdDisplay"><?php echo $adminId; ?></span></p>

<h2>Queued Calls</h2>
<div id="queued-calls-container">Loading queued calls...</div>

<hr>
<h2>WebRTC Section</h2>
<p id="call-status">No active call</p>
<button id="btnMute" onclick="toggleMute()" style="display:none;">Mute</button>
<button id="btnRequeue" onclick="requeueCall()" style="display:none;">Requeue</button>
<button id="btnEndCall" onclick="endCall()" style="display:none;">End Call</button>

<script>
// ===========================
// Global variables
// ===========================
let pc = null;
let currentCallId = null;
let localStream = null;
let isMuted = false;
let lastConnectedTime = null;
let disconnectedAt = null;
const CONNECTION_TIMEOUT = 15000; // 15 seconds
const DISCONNECT_THRESHOLD = 5000; // 5 seconds for forced reconnect

// Attempt to restore call from localStorage (in case of refresh)
if (localStorage.getItem('adminCurrentCallId')) {
    currentCallId = parseInt(localStorage.getItem('adminCurrentCallId'), 10);
}

// Periodic polling for calls list and for the active call
setInterval(fetchCalls, 3000);
setInterval(pollCurrentCall, 1000);

// On page unload, store the active call ID (if any)
window.addEventListener('beforeunload', () => {
    if (currentCallId) {
        localStorage.setItem('adminCurrentCallId', currentCallId);
    } else {
        localStorage.removeItem('adminCurrentCallId');
    }
});

// Run fetchCalls immediately at load
fetchCalls();

// If there is a currentCallId (from refresh), try to reconnect
if (currentCallId) {
    attemptReconnect();
}

// ===========================
// Functions
// ===========================

async function fetchCalls() {
    try {
        let r = await fetch('list_calls.php');
        let d = await r.json();
        renderQueuedCalls(d.queued);
    } catch (e) {
        console.error('Error fetching calls:', e);
    }
}

function renderQueuedCalls(calls) {
    let c = document.getElementById('queued-calls-container');
    c.innerHTML = '';
    if (!calls || !calls.length) {
        c.innerHTML = '<p>No queued calls</p>';
        return;
    }
    let table = document.createElement('table');
    table.border = '1';
    let thead = document.createElement('thead');
    thead.innerHTML = '<tr><th>Call ID</th><th>Phone Number</th><th>Status</th><th>Admin ID</th><th>Action</th></tr>';
    table.appendChild(thead);
    let tbody = document.createElement('tbody');
    calls.forEach(call => {
        let actionButtons = '';
        if (call.status === 'offered') {
            actionButtons = `<button onclick="adminAccept(${call.id})">Accept</button>`;
        }
        if (call.status === 'offered' || call.status === 'accepted') {
            actionButtons += ` <button onclick="endCallFromList(${call.id})">End</button>`;
        }
        let row = document.createElement('tr');
        row.innerHTML = `<td>${call.id}</td><td>${call.phone_number || 'N/A'}</td><td>${call.status}</td><td>${call.admin_id || 'None'}</td><td>${actionButtons || '---'}</td>`;
        tbody.appendChild(row);
    });
    table.appendChild(tbody);
    c.appendChild(table);
}

async function adminAccept(callId) {
    // If we are already in a call, end it first
    if (currentCallId && currentCallId !== callId) {
        endCall();
    }

    try {
        let fd = new FormData();
        fd.append('action', 'getCall');
        fd.append('call_id', callId);
        let resp = await fetch('signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            alert(data.message);
            return;
        }
        let callInfo = data.call;
        if (!callInfo.offer) {
            alert('No offer found');
            return;
        }

        currentCallId = callId;
        localStorage.setItem('adminCurrentCallId', callId);
        document.getElementById('call-status').innerText = 'Call accepted. Establishing...';
        document.getElementById('btnEndCall').style.display = 'inline-block';
        document.getElementById('btnMute').style.display = 'inline-block';
        document.getElementById('btnRequeue').style.display = 'inline-block';

        await setupLocalAudio();
        await createPeerConnection();

        let remoteDesc = new RTCSessionDescription(JSON.parse(callInfo.offer));
        await pc.setRemoteDescription(remoteDesc);

        let answer = await pc.createAnswer();
        await pc.setLocalDescription(answer);

        let fd2 = new FormData();
        fd2.append('action', 'sendAnswer');
        fd2.append('call_id', callId);
        fd2.append('answer', JSON.stringify(answer));
        await fetch('signaling.php', { method: 'POST', body: fd2 });

        lastConnectedTime = Date.now();
        document.getElementById('call-status').innerText = 'Connected to user';
    } catch (e) {
        console.error('Error accepting call:', e);
        alert('Failed to accept call: ' + e.message);
    }
}

async function setupLocalAudio() {
    try {
        localStream = await navigator.mediaDevices.getUserMedia({ audio: true, video: false });
        console.log('Admin local audio stream acquired');
    } catch (e) {
        console.error('Failed to access microphone:', e);
        alert('Microphone access denied');
        throw e;
    }
}

async function createPeerConnection() {
    if (pc) {
        pc.close();
        pc = null;
    }
    pc = new RTCPeerConnection({ iceServers: [{ urls: 'stun:stun.l.google.com:19302' }] });
    if (localStream) {
        localStream.getTracks().forEach(t => pc.addTrack(t, localStream));
        console.log('Admin added local audio tracks to peer connection');
    }

    pc.onicecandidate = ev => {
        if (ev.candidate) {
            sendIceCandidate(currentCallId, 'admin', ev.candidate);
            console.log('Admin sent ICE candidate');
        }
    };

    pc.ontrack = ev => {
        let existingAudio = document.getElementById('remoteAudio');
        if (!existingAudio) {
            let audio = document.createElement('audio');
            audio.id = 'remoteAudio';
            audio.autoplay = true;
            audio.srcObject = ev.streams[0];
            document.body.appendChild(audio);
        } else {
            existingAudio.srcObject = ev.streams[0];
        }
        console.log('Admin received user audio track');
    };

    pc.onconnectionstatechange = () => {
        console.log('Admin connection state:', pc.connectionState);
        if (pc.connectionState === 'connected') {
            disconnectedAt = null;
            lastConnectedTime = Date.now();
            document.getElementById('call-status').innerText = 'Connected to user';
        } else if (pc.connectionState === 'disconnected' || pc.connectionState === 'failed') {
            // Mark the time we lost connection
            if (!disconnectedAt) {
                disconnectedAt = Date.now();
            }
            document.getElementById('call-status').innerText = 'Connection lost';
        }
    };
}

function sendIceCandidate(callId, role, cand) {
    if (!callId) return;
    let fd = new FormData();
    fd.append('action', 'sendIceCandidate');
    fd.append('call_id', callId);
    fd.append('role', role);
    fd.append('candidate', JSON.stringify(cand));
    fetch('signaling.php', { method: 'POST', body: fd }).catch(e => console.error('ICE candidate error:', e));
}

async function pollCurrentCall() {
    if (!currentCallId || !pc) {
        return;
    }
    try {
        let fd = new FormData();
        fd.append('action', 'getCall');
        fd.append('call_id', currentCallId);
        let r = await fetch('signaling.php', { method: 'POST', body: fd });
        let d = await r.json();
        if (!d.success) return;
        let c = d.call;

        // Handle inbound ICE from user
        if (c.ice_user) {
            for (let i = 0; i < c.ice_user.length; i++) {
                try {
                    let candidate = JSON.parse(c.ice_user[i]);
                    await pc.addIceCandidate(candidate);
                    console.log('Admin added user ICE candidate');
                } catch (e) {
                    console.error('ICE candidate error:', e);
                }
            }
        }

        // If call ended, end locally
        if (c.status === 'ended') {
            console.log('Call ended detected in admin poll');
            document.getElementById('call-status').innerText = 'Call ended';
            endCall();
            return;
        }

        // Attempt reconnect if disconnected more than DISCONNECT_THRESHOLD
        if (disconnectedAt && (Date.now() - disconnectedAt) > DISCONNECT_THRESHOLD) {
            console.log('Admin attempting to reconnect...');
            attemptReconnect();
            return;
        }

        // If connected but no activity for 15s, end call
        if ((pc.connectionState === 'disconnected' || pc.connectionState === 'failed') && lastConnectedTime) {
            if ((Date.now() - lastConnectedTime) > CONNECTION_TIMEOUT) {
                console.log('Connection timeout detected in admin poll');
                document.getElementById('call-status').innerText = 'Connection lost for too long, ending call';
                endCall();
            }
        }
    } catch (e) {
        console.error('Poll error:', e);
    }
}

async function attemptReconnect() {
    // Attempting the same logic as if we just accepted the call
    // so that we can re-do the handshake.

    if (!currentCallId) return;

    document.getElementById('call-status').innerText = 'Reconnecting...';
    document.getElementById('btnEndCall').style.display = 'inline-block';
    document.getElementById('btnMute').style.display = 'inline-block';
    document.getElementById('btnRequeue').style.display = 'inline-block';

    try {
        await setupLocalAudio();
        await createPeerConnection();

        // Fetch the call info again
        let fd = new FormData();
        fd.append('action', 'getCall');
        fd.append('call_id', currentCallId);
        let resp = await fetch('signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            console.error('Reconnection getCall failed:', data.message);
            return;
        }
        let callInfo = data.call;

        if (callInfo.offer) {
            let remoteDesc = new RTCSessionDescription(JSON.parse(callInfo.offer));
            await pc.setRemoteDescription(remoteDesc);
            let answer = await pc.createAnswer();
            await pc.setLocalDescription(answer);

            let fd2 = new FormData();
            fd2.append('action', 'sendAnswer');
            fd2.append('call_id', currentCallId);
            fd2.append('answer', JSON.stringify(answer));
            await fetch('signaling.php', { method: 'POST', body: fd2 });
        }
        disconnectedAt = null;
        lastConnectedTime = Date.now();
        document.getElementById('call-status').innerText = 'Reconnected';
    } catch (e) {
        console.error('Reconnection error:', e);
        document.getElementById('call-status').innerText = 'Reconnection failed';
    }
}

async function requeueCall() {
    if (!currentCallId) return;
    try {
        let fd = new FormData();
        fd.append('action', 'requeueCall');
        fd.append('call_id', currentCallId);
        let resp = await fetch('signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            alert('Failed to requeue call: ' + data.message);
            return;
        }
        // Clean up local stuff
        if (pc) {
            pc.close();
            pc = null;
        }
        if (localStream) {
            localStream.getTracks().forEach(track => track.stop());
            localStream = null;
        }
        let remoteAudio = document.getElementById('remoteAudio');
        if (remoteAudio) {
            remoteAudio.remove();
        }
        currentCallId = null;
        localStorage.removeItem('adminCurrentCallId');
        lastConnectedTime = null;
        disconnectedAt = null;
        document.getElementById('call-status').innerText = 'Call requeued';
        document.getElementById('btnEndCall').style.display = 'none';
        document.getElementById('btnMute').style.display = 'none';
        document.getElementById('btnRequeue').style.display = 'none';
    } catch (e) {
        console.error('Requeue error:', e);
        alert('Failed to requeue call: ' + e.message);
    }
}

async function endCallFromList(callId) {
    try {
        let fd = new FormData();
        fd.append('action', 'endCall');
        fd.append('call_id', callId);
        let resp = await fetch('signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            alert('Failed to end call: ' + data.message);
            return;
        }
        if (callId === currentCallId) {
            endCall();
        } else {
            fetchCalls(); // Refresh the call list
        }
    } catch (e) {
        console.error('End call error:', e);
        alert('Failed to end call: ' + e.message);
    }
}

function endCall() {
    console.log('Admin ending call');
    if (currentCallId) {
        let fd = new FormData();
        fd.append('action', 'endCall');
        fd.append('call_id', currentCallId);
        fetch('signaling.php', { method: 'POST', body: fd })
            .then(resp => resp.json())
            .then(data => {
                if (!data.success) {
                    console.error('End call failed:', data.message);
                }
            })
            .catch(e => console.error('End call error:', e));
    }
    if (pc) {
        pc.close();
        pc = null;
    }
    if (localStream) {
        localStream.getTracks().forEach(track => track.stop());
        localStream = null;
    }
    let remoteAudio = document.getElementById('remoteAudio');
    if (remoteAudio) {
        remoteAudio.remove();
    }
    currentCallId = null;
    localStorage.removeItem('adminCurrentCallId');
    lastConnectedTime = null;
    disconnectedAt = null;
    document.getElementById('call-status').innerText = 'No active call';
    document.getElementById('btnEndCall').style.display = 'none';
    document.getElementById('btnMute').style.display = 'none';
    document.getElementById('btnRequeue').style.display = 'none';
}

async function toggleMute() {
    if (!localStream) return;
    isMuted = !isMuted;
    localStream.getAudioTracks().forEach(t => t.enabled = !isMuted);
    document.getElementById('btnMute').innerText = isMuted ? 'Unmute' : 'Mute';

    let fd = new FormData();
    fd.append('action', 'muteCall');
    fd.append('call_id', currentCallId);
    fd.append('is_muted', isMuted ? '1' : '0');
    await fetch('signaling.php', { method: 'POST', body: fd }).catch(e => console.error('Mute error:', e));
}
</script>
</body>
</html>
