<?php
session_start();
// Assign user ID if not set
if (!isset($_SESSION['user_id'])) {
    $_SESSION['user_id'] = uniqid('user_', true);
}
$userId = $_SESSION['user_id'];

// Read .env for ADMIN_DOMAIN
$envPath = __DIR__ . '/.env';
$adminDomain = '';
if (file_exists($envPath)) {
    $lines = file($envPath, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    foreach ($lines as $l) {
        if (strpos($l, 'ADMIN_DOMAIN=') === 0) {
            $adminDomain = trim(substr($l, strlen('ADMIN_DOMAIN=')));
        }
    }
}
if (!$adminDomain) {
    $adminDomain = 'http://localhost/admin';
}
?>
<!DOCTYPE html>
<html>
<head>
    <title>User Call Center</title>
    <style>
        table { border-collapse: collapse; width: 100%; }
        th, td { border: 1px solid #ccc; padding: 8px; text-align: left; }
        th { background-color: #f2f2f2; }
    </style>
</head>
<body>
<h1>User Call Center</h1>
<p>Your User ID: <?php echo $userId; ?></p>

<div id="phone-section" style="border:1px solid #ccc; padding:1em;">
    <label for="phoneInput">Phone Number:</label>
    <input type="tel" id="phoneInput" placeholder="Enter your phone number"><br><br>
    <button onclick="startCall()">Continue</button>
</div>

<div id="call-section" style="display:none;">
    <button id="btnMute" onclick="toggleMute()" style="display:none;">Mute</button>
    <button id="btnEndCall" onclick="endCall()" style="display:none;">End Call</button>
    <div id="call-status"></div>
</div>

<script>
// ===========================
// Global Variables
// ===========================
const adminDomain = '<?php echo $adminDomain; ?>';
let userId = '<?php echo $userId; ?>';
let callId = null;
let pc = null;
let localStream = null;
let remoteAudio = null;
let phoneNumber = '';
let holdAudio = null;
let isMuted = false;
let lastConnectedTime = null;
let disconnectedAt = null;

const CONNECTION_TIMEOUT = 15000; // 15 seconds to declare the call dead
const DISCONNECT_THRESHOLD = 5000; // 5 seconds to attempt reconnection

// Attempt to restore call from localStorage (in case of refresh)
if (localStorage.getItem('userCurrentCallId')) {
    callId = parseInt(localStorage.getItem('userCurrentCallId'), 10);
}

// Polling
setInterval(pollCallInfo, 1000);

window.addEventListener('beforeunload', () => {
    if (callId) {
        localStorage.setItem('userCurrentCallId', callId);
    } else {
        localStorage.removeItem('userCurrentCallId');
    }
});

if (callId) {
    // If we have a call ID from refresh, display call section & attempt reconnect
    document.getElementById('phone-section').style.display = 'none';
    document.getElementById('call-section').style.display = 'block';
    document.getElementById('btnEndCall').style.display = 'inline-block';
    document.getElementById('btnMute').style.display = 'inline-block';
    document.getElementById('call-status').innerText = 'Reconnecting...';
    attemptReconnect();
}

// ===========================
// Functions
// ===========================

async function startCall() {
    phoneNumber = document.getElementById('phoneInput').value.trim();
    if (!phoneNumber) {
        alert('Please enter your phone number');
        return;
    }
    document.getElementById('phone-section').style.display = 'none';
    document.getElementById('call-section').style.display = 'block';

    await createCall();
}

async function createCall() {
    let fd = new FormData();
    fd.append('action', 'createCall');
    fd.append('user_id', userId);
    fd.append('phone_number', phoneNumber);

    try {
        let resp = await fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            alert('Cannot create call: ' + data.message);
            document.getElementById('call-section').style.display = 'none';
            document.getElementById('phone-section').style.display = 'block';
            return;
        }
        callId = data.call_id;
        localStorage.setItem('userCurrentCallId', callId);

        document.getElementById('call-status').innerText = 'Call created. Preparing offer...';
        document.getElementById('btnEndCall').style.display = 'inline-block';
        document.getElementById('btnMute').style.display = 'inline-block';

        await setupLocalAudio();
        await createPeerConnection();

        let offer = await pc.createOffer();
        await pc.setLocalDescription(offer);

        let fd2 = new FormData();
        fd2.append('action', 'sendOffer');
        fd2.append('call_id', callId);
        fd2.append('offer', JSON.stringify(offer));
        await fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd2 });

        document.getElementById('call-status').innerText = 'Waiting for admin to join...';
        playHoldMusic();
    } catch (e) {
        console.error('Create call error:', e);
        alert('Failed to initiate call: ' + e.message);
        endCall();
    }
}

async function setupLocalAudio() {
    try {
        localStream = await navigator.mediaDevices.getUserMedia({ audio: true, video: false });
        console.log('User local audio stream acquired');
    } catch (e) {
        console.error('Failed to access microphone:', e);
        alert('Microphone access denied');
        throw e;
    }
}

async function createPeerConnection() {
    if (pc) {
        pc.close();
        pc = null;
    }
    pc = new RTCPeerConnection({ iceServers: [{ urls: 'stun:stun.l.google.com:19302' }] });
    if (localStream) {
        localStream.getTracks().forEach(t => pc.addTrack(t, localStream));
        console.log('User added local audio tracks to peer connection');
    }

    pc.onicecandidate = ev => {
        if (ev.candidate) {
            sendIceCandidate(callId, 'user', ev.candidate);
            console.log('User sent ICE candidate');
        }
    };

    pc.ontrack = ev => {
        if (!remoteAudio) {
            remoteAudio = document.createElement('audio');
            remoteAudio.autoplay = true;
            remoteAudio.id = 'remoteAudio';
            document.body.appendChild(remoteAudio);
        }
        remoteAudio.srcObject = ev.streams[0];
        console.log('User received admin audio track');
    };

    pc.onconnectionstatechange = () => {
        console.log('User connection state:', pc.connectionState);
        if (pc.connectionState === 'connected') {
            disconnectedAt = null;
            lastConnectedTime = Date.now();
            document.getElementById('call-status').innerText = 'Connected to Admin';
            stopHoldMusic();
        } else if (pc.connectionState === 'disconnected' || pc.connectionState === 'failed') {
            if (!disconnectedAt) {
                disconnectedAt = Date.now();
            }
            document.getElementById('call-status').innerText = 'Connection lost';
            playHoldMusic();
        }
    };
}

function sendIceCandidate(cid, role, cand) {
    if (!cid) return;
    let fd = new FormData();
    fd.append('action', 'sendIceCandidate');
    fd.append('call_id', cid);
    fd.append('role', role);
    fd.append('candidate', JSON.stringify(cand));
    fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd }).catch(e => console.error('ICE candidate error:', e));
}

function playHoldMusic() {
    if (!holdAudio) {
        holdAudio = new Audio('hold.mp3');
        holdAudio.loop = true;
    }
    holdAudio.play().catch(e => {
        console.error('Hold music playback error:', e);
        document.getElementById('call-status').innerText += ' (Hold music failed to play)';
    });
}

function stopHoldMusic() {
    if (holdAudio) {
        holdAudio.pause();
        holdAudio.currentTime = 0;
    }
}

async function pollCallInfo() {
    if (!callId) return;
    let fd = new FormData();
    fd.append('action', 'getCall');
    fd.append('call_id', callId);
    try {
        let r = await fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd });
        let d = await r.json();
        if (!d.success) {
            console.error('Poll call info failed:', d.message);
            return;
        }
        let c = d.call;

        // If call ended, end it locally
        if (c.status === 'ended') {
            console.log('Call ended detected in user poll');
            document.getElementById('call-status').innerText = 'Call ended';
            endCall();
            return;
        }

        // If call is queued or requeued, we re-send an offer
        if ((c.status === 'queued' || (c.status === 'offered' && !c.admin_id)) && pc) {
            if (pc.signalingState !== 'have-local-offer') {
                console.log('Re-sending offer for queued/requeued call');
                await recreateOffer();
            }
            document.getElementById('call-status').innerText = 'On hold, waiting for admin...';
            playHoldMusic();
        }

        // If an answer exists and our state is still have-local-offer, apply it
        if (c.answer && pc && pc.signalingState === 'have-local-offer') {
            console.log('Applying admin answer');
            let rd = new RTCSessionDescription(JSON.parse(c.answer));
            await pc.setRemoteDescription(rd);
            document.getElementById('call-status').innerText = 'Connected to Admin';
            stopHoldMusic();
        }

        // Inbound ICE from admin
        if (c.ice_admin && pc) {
            for (let s of c.ice_admin) {
                try {
                    let cand = JSON.parse(s);
                    await pc.addIceCandidate(cand);
                    console.log('User added admin ICE candidate');
                } catch (err) {
                    console.error('Failed to add admin ICE:', err);
                }
            }
        }

        // If call is accepted
        if (c.status === 'accepted') {
            document.getElementById('call-status').innerText = 'Connected to Admin';
            stopHoldMusic();
            if ((pc.connectionState === 'disconnected' || pc.connectionState === 'failed') && lastConnectedTime) {
                // If no real connection for more than the threshold, try reconnect
                if (!disconnectedAt) {
                    disconnectedAt = Date.now();
                }
                if ((Date.now() - disconnectedAt) > DISCONNECT_THRESHOLD) {
                    console.log('User attempting to reconnect...');
                    attemptReconnect();
                } else if ((Date.now() - lastConnectedTime) > CONNECTION_TIMEOUT) {
                    console.log('Connection timeout detected in user poll');
                    document.getElementById('call-status').innerText = 'Connection lost for too long, ending call';
                    endCall();
                }
            } else {
                // Keep track of a valid connection
                lastConnectedTime = Date.now();
            }
        }
    } catch (e) {
        console.error('Poll error:', e);
    }
}

async function recreateOffer() {
    if (!pc) return;
    await pc.setLocalDescription(await pc.createOffer());
    let fd2 = new FormData();
    fd2.append('action', 'sendOffer');
    fd2.append('call_id', callId);
    fd2.append('offer', JSON.stringify(pc.localDescription));
    await fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd2 }).catch(e => console.error('Offer error:', e));
}

async function attemptReconnect() {
    document.getElementById('call-status').innerText = 'Reconnecting...';
    try {
        await setupLocalAudio();
        await createPeerConnection();

        // Re-send an offer if not accepted yet
        let fd = new FormData();
        fd.append('action', 'getCall');
        fd.append('call_id', callId);
        let resp = await fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd });
        let data = await resp.json();
        if (!data.success) {
            console.error('Reconnection getCall failed:', data.message);
            return;
        }
        let c = data.call;

        if (c.status === 'accepted' && c.answer) {
            // If there's already an answer, apply it
            console.log('Applying admin answer in reconnection');
            let rd = new RTCSessionDescription(JSON.parse(c.answer));
            await pc.setRemoteDescription(rd);
            document.getElementById('call-status').innerText = 'Connected to Admin';
            stopHoldMusic();
        } else {
            // Otherwise, we need to re-offer
            console.log('Re-offering for reconnection');
            await recreateOffer();
        }

        disconnectedAt = null;
        lastConnectedTime = Date.now();
        console.log('User reconnected');
    } catch (e) {
        console.error('Reconnection error:', e);
        document.getElementById('call-status').innerText = 'Reconnection failed';
    }
}

function endCall() {
    console.log('User ending call');
    stopHoldMusic();
    if (callId) {
        let fd = new FormData();
        fd.append('action', 'endCall');
        fd.append('call_id', callId);
        fetch(adminDomain + '/signaling.php', { method: 'POST', body: fd })
            .then(resp => resp.json())
            .then(data => {
                if (!data.success) {
                    console.error('End call failed:', data.message);
                }
            })
            .catch(e => console.error('End call error:', e));
    }
    if (pc) {
        pc.close();
        pc = null;
    }
    if (localStream) {
        localStream.getTracks().forEach(track => track.stop());
        localStream = null;
    }
    if (remoteAudio) {
        remoteAudio.remove();
        remoteAudio = null;
    }
    callId = null;
    localStorage.removeItem('userCurrentCallId');
    lastConnectedTime = null;
    disconnectedAt = null;
    document.getElementById('call-status').innerText = 'Call ended';
    document.getElementById('btnEndCall').style.display = 'none';
    document.getElementById('btnMute').style.display = 'none';
    document.getElementById('call-section').style.display = 'none';
    document.getElementById('phone-section').style.display = 'block';
    document.getElementById('phoneInput').value = '';
}

async function toggleMute() {
    if (!localStream) return;
    isMuted = !isMuted;
    localStream.getAudioTracks().forEach(t => t.enabled = !isMuted);
    document.getElementById('btnMute').innerText = isMuted ? 'Unmute' : 'Mute';
}
</script>
</body>
</html>
